@@ -4,4 +4,4 @@
 #' @slot pointer externalptr object
 #' @keywords internal
 
-setClass( "chunker", representation( pointer = "externalptr" ) )
\ No newline at end of file
+setClass( "chunker", representation( pointer = "externalptr", attr = "list"))
\ No newline at end of file
