@@ -4,23 +4,26 @@
 #' @keywords internal
 
 setMethod( "initialize", "chunker", 
-           function(.Object, path_, sep_, has_colnames_, has_rownames_, 
-                    chunksize_, data_format_,  columns_classes_) {
+           function(.Object, path_, sep_, quoted_, has_colnames_, 
+                    has_rownames_, chunksize_, data_format_,  columns_classes_) {
              
              path_ <- normalizePath(path_)
              
              if(data_format_ == "matrix") {
                .Object@pointer  <- chunker__new_matrix(path_, 
                                                       sep_, 
+                                                      quoted_,
                                                       has_colnames_, 
                                                       has_rownames_, 
                                                       chunksize_)
              } else {
-               .Object@pointer  <- chunker__new_data_frame(path_, sep_,
-                                                          has_colnames_, 
-                                                          has_rownames_,
-                                                          chunksize_, 
-                                                          columns_classes_)
+               .Object@pointer  <- chunker__new_data_frame(path_, 
+                                                           sep_,
+                                                           quoted_,
+                                                           has_colnames_, 
+                                                           has_rownames_,
+                                                           chunksize_, 
+                                                           columns_classes_)
              }
              .Object
            })
@@ -33,7 +36,7 @@ setMethod( "initialize", "chunker",
 #' \item{\bold{get_table}}{: retrieve the current data chunk contained in the object}
 #' }
 #' 
-#' In addition, this information can be retrieved from chunker objects:
+#' In addition, the following information can be retrieved from chunker objects:
 #' \enumerate{
 #' \item{\bold{get_completed}}{: get the number of rows already read}
 #' \item{\bold{get_colnames}}{: get column names of the chunker object}
@@ -116,6 +119,21 @@ setMethod("get_completed", "chunker", function(obj) {
   chunker__get_completed(obj@pointer)
 })
 
+#' get_total
+#' @name get_total
+#' @description NULL
+#' @rdname chunker-methods
+
+setGeneric("get_total", function(obj) standardGeneric("get_total"))
+
+
+#' @aliases get_total,chunker-methods
+#' @rdname chunker-methods
+
+setMethod("get_total", "chunker", function(obj) {
+  chunker__get_total(obj@pointer)
+})
+
 
 #' get_type
 #' @name get_type
@@ -133,6 +151,39 @@ setMethod("get_type", "chunker", function(obj) {
   chunker__get_type(obj@pointer)
 })
 
+#' get_attr
+#' @name get_attr
+#' @description NULL
+#' @rdname chunker-methods
+#' @export
+
+setGeneric("get_attr", function(obj) standardGeneric("get_attr"))
+
+
+#' @aliases get_attr,chunker-methods
+#' @rdname chunker-methods
+
+setMethod("get_attr", "chunker", function(obj) {
+  obj@attr
+})
+
+#' @aliases print,chunker-methods
+#' @keywords internal
+
+setMethod("show", "chunker", function(object) {
+  total_lines <- get_total(object)
+  completed_lines <- get_completed(object)
+  cat("\n>---- chunker object ---->\n\n")
+  cat("Path: ", object@attr$path, "\n")
+  cat("Chunk size: ", object@attr$chunksize, "| Sep: ", deparse(object@attr$sep), "| Quoted: ", object@attr$quoted, "\n")
+  cat("Colnames: ", object@attr$has_colnames, "| Rownames: ", object@attr$has_rownames, "\n")
+  cat("Data imported as: ", object@attr$data_format, "\n")
+  cat("Total lines in file: ", ifelse (object@attr$data_format == "data.frame", total_lines, "unknown"), "\n")
+  cat("Lines completed: ", completed_lines)
+  cat(" ", ifelse (object@attr$data_format == "data.frame", paste0("(", round(100 * completed_lines/total_lines), "%)\n"), "\n"))
+  cat("Additional information: use the function 'get_attr' with this object", "\n\n")
+})
+
 
 #-------------
 # Add documentation for matrix2df
