@@ -14,8 +14,10 @@ using namespace Rcpp;
 
 StringMatrix NULLstringm(0);
 const StringMatrix empty_stringm = NULLstringm;
-DataFrame NULLdf(0);
+
+DataFrame NULLdf = DataFrame::create();
 const DataFrame empty_df = NULLdf;
+
 std::vector<std::string> autovector;
 #define auto_vector autovector
 
@@ -29,17 +31,18 @@ namespace _chunkR {
 //' chunker, matrix-constructor
 //' @keywords internal
 
-chunker::chunker(const std::string path, char sep, bool has_colnames, bool has_rownames, size_t chunksize) :
-path(path), sep(sep), has_colnames(has_colnames),
+chunker::chunker(const std::string path, char sep, bool quoted, 
+                 bool has_colnames, bool has_rownames, size_t chunksize) :
+path(path), sep(sep), quoted(quoted), has_colnames(has_colnames),
 has_rownames(has_rownames), chunksize(chunksize), output_format("matrix"), n_row(0), n_col(0),
 rnames([&chunksize] {std::vector<std::string> out; out.reserve(chunksize); return out;}()), 
 cnames([] {std::vector<std::string> out; return out;}()), 
 pointer_position(0), line(new std::string), element(new std::string), 
-lines_completed(0), word(auto_vector) {
+lines_completed(0), word(auto_vector), validation_state("") {
   
+  set_offset();
   data_chunk.m = empty_stringm;
   data_chunk.df = empty_df; 
-  // configure for data frames
   
   Rcout << "New chunker object\n";
   Rcout << "Path: " << path << std::endl;
@@ -50,27 +53,24 @@ lines_completed(0), word(auto_vector) {
 //' chunker, dataframe-constructor
 //' @keywords internal
 
-chunker::chunker(const std::string path, char sep, bool has_colnames, bool has_rownames, size_t chunksize,
-               StringVector column_types) :
-path(path), sep(sep), has_colnames(has_colnames),
+chunker::chunker(const std::string path, char sep, bool quoted, 
+                 bool has_colnames, bool has_rownames, size_t chunksize,
+                 StringVector column_types) :
+path(path), sep(sep),  quoted(quoted), has_colnames(has_colnames),
 has_rownames(has_rownames), chunksize(chunksize), output_format("data.frame"), n_row(0), n_col(0),
 rnames([&chunksize] {std::vector<std::string> out; out.reserve(chunksize); return out;}()), 
 cnames([] {std::vector<std::string> out; return out;}()), 
 pointer_position(0), line(new std::string), element(new std::string), 
-lines_completed(0), word(auto_vector) {
+lines_completed(0), word(auto_vector), validation_state("") {
   
+  // set offset
+  set_offset();
   // count lines in file
-  Rcout << "Counting lines in file... ";
-  std::ifstream this_file(path); 
-  n_lines = std::count(std::istreambuf_iterator<char>(this_file), 
-                       std::istreambuf_iterator<char>(), '\n');
-  if(has_colnames) {
-    n_lines = n_lines -1;
-  }
-  Rcout << "done.\n";
+  count_lines();
   
   data_chunk.m = empty_stringm;
   data_chunk.df = empty_df; 
+  
   // configure for data frames
   size_t df_size = column_types.size();
   if(df_size > 0) {
@@ -104,7 +104,6 @@ chunker::~chunker() {
   delete element;
 }
 
-
 //--------------------------------------------
 // NEXT_CHUNK 
 //--------------------------------------------
@@ -125,27 +124,34 @@ bool chunker::next_chunk() {
 //' @keywords internal
 
 bool chunker::next_chunk_matrix() {
-  
+  if(!is_valid_chunker()) {
+    Rcout << validation_state << std::endl;
+    return false;
+  }
   if (!line_container.eof()) {
     
-    line_container.open(path, std::ios::binary);
+    line_container.open(path.c_str(), std::ios::binary);
     if (line_container.fail()) {
       std::ostringstream msg;
       msg << "Input file opening failed.\n";
       throw(msg.str());
     }
     
-    line_container.seekg(pointer_position);
+    line_container.seekg(pointer_position + offset);
     
     // for first line read before
     size_t lines_read_chunk = 0;
     size_t initial_lines  = lines_completed;
     
-    while (std::getline(line_container, *line)) {
+    while (std::getline(line_container, *line, eof)) {
       bool is_name = true;
-      
       std::stringstream ss(*line);
+
       while (std::getline(ss, *element, sep)) {
+	
+        if(quoted) {
+          element->erase(remove(element->begin(), element->end(), '\"' ), element->end());
+        }
         if (is_name && has_rownames) {
           rnames.push_back(*element);
           is_name = false;
@@ -159,10 +165,25 @@ bool chunker::next_chunk_matrix() {
       if (++lines_read_chunk >= chunksize) {
         break;
       }
+      line_container.seekg(offset, std::ios::cur);
     }
     
+
     pointer_position = line_container.tellg();
     
+    try {
+      if(word.size() != (lines_read_chunk * n_col)) throw(1);
+    } catch(int e) {
+      Rcout << "Number of elements are not multiple of matrix dimension" << std::endl;
+      line_container.seekg(0, std::ios::end);
+      word.clear();
+      line_container.close();
+      rnames.clear();
+      data_chunk.m = empty_stringm;
+      validation_state = "Invalid object: number of elements are not multiple of matrix dimension";
+      return false;
+    }
+    
     // create output
     StringMatrix output(lines_read_chunk, n_col);
     int k = 0;
@@ -191,8 +212,7 @@ bool chunker::next_chunk_matrix() {
     line_container.close();
     
     if (output.nrow() == 0) {
-      StringMatrix output(0);
-      data_chunk.m = output;
+      data_chunk.m = empty_stringm;
       return false;
     }
     
@@ -200,8 +220,7 @@ bool chunker::next_chunk_matrix() {
     return true;
     
   } else {
-    StringMatrix output(0);
-    data_chunk.m = output;
+    data_chunk.m = empty_stringm;
     return false;
   }
   
@@ -214,14 +233,14 @@ bool chunker::next_chunk_df() {
   
   if (!line_container.eof()) {
     
-    line_container.open(path, std::ios::binary);
+    line_container.open(path.c_str(), std::ios::binary);
     if (line_container.fail()) {
       std::ostringstream msg;
       msg << "Input file opening failed.\n";
       throw(msg.str());
     }
     
-    line_container.seekg(pointer_position);
+    line_container.seekg(pointer_position + offset);
     
     // for first line read before
     size_t lines_read_chunk = 0;
@@ -234,20 +253,27 @@ bool chunker::next_chunk_df() {
       output = mixed_list(col_types, n_lines - initial_lines); 
     }
     
-    while (std::getline(line_container, *line)) {
+    while (std::getline(line_container, *line, eof)) {
       bool is_name = true;
       
       std::stringstream ss(*line);
       int count_elements = 0;
       while (std::getline(ss, *element, sep)) {
         if (is_name && has_rownames) {
+          if(quoted) {
+            element->erase(remove(element->begin(), element->end(), '\"' ), element->end());
+          }
           rnames.push_back(*element);
           is_name = false;
           continue;
         }
         
         if(col_types[count_elements] == 0) {
-          std::string temp_out = * element;
+	
+          if(quoted) {
+            element->erase(remove(element->begin(), element->end(), '\"' ), element->end());
+          }
+          std::string temp_out = *element;
           (Rcpp::as<StringVector>(output[count_elements]))[lines_read_chunk] = temp_out;
         } else if(col_types[count_elements] == 1) {
           double temp_out = std::atof((* element).c_str());
@@ -264,6 +290,8 @@ bool chunker::next_chunk_df() {
       if (++lines_read_chunk >= chunksize) {
         break;
       }
+      
+      line_container.seekg(offset, std::ios::cur);
     }
     
     pointer_position = line_container.tellg();
@@ -289,7 +317,7 @@ bool chunker::next_chunk_df() {
     
     if (lines_read_chunk == 0) {
       DataFrame output;
-      data_chunk.df = output;
+      data_chunk.df = empty_df;
       return false;
     }
     
@@ -297,8 +325,7 @@ bool chunker::next_chunk_df() {
     return true;
     
   } else {
-    DataFrame output;
-    data_chunk.df = output;
+    data_chunk.df = empty_df;
     return false;
   }
   
@@ -314,7 +341,7 @@ bool chunker::next_chunk_df() {
 
 void chunker::set_colnames() {
   
-  line_container.open(path, std::ios::binary);
+  line_container.open(path.c_str(), std::ios::binary);
   try {
     if (line_container.fail()) {
       std::ostringstream msg;
@@ -326,9 +353,13 @@ void chunker::set_colnames() {
   }
   
   if (has_colnames) {
-    std::getline(line_container, *line);
+    std::getline(line_container, *line, eof);
     std::stringstream headss(*line);
     while (std::getline(headss, *element, sep)) {
+	
+      if(quoted) {
+        element->erase(remove(element->begin(), element->end(), '\"' ), element->end());
+      }
       cnames.push_back(*element);
     }
   }
@@ -336,7 +367,7 @@ void chunker::set_colnames() {
   
   // Read first line only to set number of columns
   bool is_name = true;
-  std::getline(line_container, *line);
+  std::getline(line_container, *line, eof);
   std::stringstream ss(*line);
   while (std::getline(ss, *element, sep)) {
     if (is_name && has_rownames) {
@@ -374,11 +405,11 @@ void chunker::set_colnames() {
 //' set_generic_rownames
 //' @keywords internal
 
-std::vector<std::string> chunker::set_generic_rownames(std::string what, size_t start_from, size_t n_row) {
+std::vector<std::string> chunker::set_generic_rownames(std::string what, size_t start_from, size_t rownumber) {
   std::ostringstream os;
   std::vector<std::string> output;
-  output.reserve(n_row);
-  for (size_t i = start_from; i < start_from + n_row; ++i) {
+  output.reserve(rownumber);
+  for (size_t i = start_from; i < start_from + rownumber; ++i) {
     os << what << "_" << i;
     output.push_back(os.str());
     os.str("");
@@ -390,11 +421,13 @@ std::vector<std::string> chunker::set_generic_rownames(std::string what, size_t
 //' set_generic_colnames
 //' @keywords internal
 
-std::vector<std::string> chunker::set_generic_colnames(std::string what,  size_t start_from, size_t n_col) {
+std::vector<std::string> chunker::set_generic_colnames(std::string what,  
+                                                       size_t start_from,
+                                                       size_t colnumber) {
   std::ostringstream os;
   std::vector<std::string> output;
-  output.reserve(n_col);
-  for (size_t i = start_from; i < start_from + n_col; ++i) {
+  output.reserve(colnumber);
+  for (size_t i = start_from; i < start_from + colnumber; ++i) {
     os << what << "_" << i;
     output.push_back(os.str());
     os.str("");
@@ -413,6 +446,9 @@ std::vector<std::string> chunker::set_generic_colnames(std::string what,  size_t
 //'@keywords internal
 
 StringMatrix chunker::get_matrix() {
+  if(!is_valid_chunker()) {
+    Rcpp::stop(validation_state); 
+  }
   return data_chunk.m;
 } 
 
@@ -420,6 +456,9 @@ StringMatrix chunker::get_matrix() {
 //'@keywords internal
 //'
 DataFrame chunker::get_dataframe() {
+  if(!is_valid_chunker()) {
+    Rcpp::stop(validation_state); 
+  }
   return data_chunk.df;
 } 
 
@@ -442,6 +481,13 @@ size_t chunker::get_completed() {
   return lines_completed;
 }
 
+//' chunker__get_completed
+//' @keywords internal
+
+size_t chunker::get_total() {
+  return n_lines;
+}
+
 //' chunker__get_type
 //' @keywords internal
 
@@ -460,7 +506,7 @@ const std::string chunker::get_type() {
 inline List chunker::mixed_list(std::vector<int> x,  int howmuch) {
   List output;
   
-  for(int i = 0; i < x.size(); ++i) {
+  for(size_t i = 0; i < x.size(); ++i) {
     switch (x[i]) {
     case 0:
   {
@@ -493,56 +539,68 @@ inline List chunker::mixed_list(std::vector<int> x,  int howmuch) {
   return output;
 }
 
+// to create an invalid object, passa text to validation state.
+// If validation_state.size() != 0, throw an error
+bool chunker::is_valid_chunker() {
+  if(validation_state.size() != 0) return false;
+  return true;
+}
 
-// //--------------------------------------------
-// // MODULE
-// //--------------------------------------------
-// 
-// //' validatos
-// 
-// bool validate_C1(SEXP* args, int nargs) {
-//   if( nargs != 6 ) return false;
-//   return true;
+void chunker::set_offset() {
+  std::ifstream file(path.c_str(), std::ios::binary);
+  char c;
+  offset = -1;
+  while (file.get(c)) {
+    if (c == '\n') {
+      offset = 0;
+      eof = '\n';
+      break;
+    }
+    if (c == '\r') {
+      if (file.get(c)) {
+        if (c == '\n') {
+          offset = 1;
+          eof = '\r';
+          break;
+        }
+      }
+      offset = 0;
+      eof = '\r';
+      break;
+    }
+  }
+  if(offset == -1) {
+    Rcpp::stop("Invalid end of line");
+  }
+}
+
+
+void chunker::count_lines() {
+Rcout << "Counting lines in file... ";
+std::ifstream this_file(path.c_str(), std::ios::binary); 
+n_lines = std::count(std::istreambuf_iterator<char>(this_file), 
+                     std::istreambuf_iterator<char>(), eof);
+this_file.close();
+if(has_colnames) {
+  n_lines = n_lines -1;
+}
+Rcout << "done.\n";
+}
+
+// bool is_space(char x) {
+//   if(x == ' ' || x == '\n' || x == '\n' || x == '\v' || x == '\f' || x == '\r') {
+//     return true;
+//   } 
+//   return false;
 // }
 // 
-// bool validate_C2(SEXP* args, int nargs) {
-//   if( nargs != 5 ) return false;
-//   return true;
-// }
-// 
-// //' chunker_module
-// 
-// RCPP_MODULE(chunker_module) {
-//   class_<chunker>("chunker")
-//   .constructor<std::string, char, bool, bool, size_t, StringVector >("data.frame constructor", &validate_C1)
-//   .constructor<std::string, char, bool, bool, size_t>("matrix constructor", &validate_C2)
-//   .method("get_colnames", &chunker::get_colnames)
-//   .method("next_chunk", &chunker::next_chunk)
-//   .method("get_matrix", &chunker::get_matrix)
-//   .method("get_dataframe", &chunker::get_dataframe)
-//   //.method("get_matrix2dataframe", &chunker::get_matrix2dataframe) DEPRECATED IN chunkR 1.1.0
-//   .method("get_completed", &chunker::get_completed)
-//   .method("get_type", &chunker::get_completed)
-//   ;
-// }
-
-//--------------------------------------------
-// DEPRECATED
-//--------------------------------------------
-
-// DEPRECATED IN chunkR 1.1.0
-// //' chunker__get_dataframe
-// //' @keywords internal
-// 
-// DataFrame chunker::get_matrix2dataframe() {
-//   List output(n_col);
-//   for (size_t i = 0; i < n_col; ++i) {
-//     output[i] = data_chunk.m(_, i);
+// bool test_empty(std::string buffer) {
+//   for (int i = 0; i< buffer.length(); ++i){
+//     if(!is_space(buffer[i])) {
+//       return false;
+//     } 
+//     return true;
 //   }
-//   output.attr("row.names") = rownames(data_chunk.m);
-//   output.attr("names") = colnames(data_chunk.m);
-//   output.attr("class") = "data.frame";
-//   return output;
 // }
 
 
